"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CachedStorageStrategy = void 0;
var tslib_1 = require("tslib");
var base_storage_1 = require("../base-storage");
var constants_1 = require("../utils/constants");
var local_storage_1 = require("./local-storage");
var null_storage_1 = require("./null-storage");
var wix_storage_1 = require("./wix-storage");
var DELETED = '___DELETED___';
var CachedStorageStrategy = /** @class */ (function (_super) {
    tslib_1.__extends(CachedStorageStrategy, _super);
    function CachedStorageStrategy(_a) {
        var remoteStrategy = _a.remoteStrategy, localStrategy = _a.localStrategy;
        var _this = _super.call(this) || this;
        if (shouldIgnoreCache(remoteStrategy)) {
            localStrategy = new null_storage_1.NullStrategy();
        }
        _this.remoteStrategy = base_storage_1.BaseStorage.verify(remoteStrategy);
        _this.localStrategy = base_storage_1.BaseStorage.verify(localStrategy || new local_storage_1.LocalStorageStrategy());
        return _this;
    }
    CachedStorageStrategy.prototype.extendScope = function (scope) {
        return this.remoteStrategy.extendScope(scope);
    };
    CachedStorageStrategy.prototype._cacheItem = function (key, value, options) {
        return this.localStrategy.setItem(key, value, Object.assign(options, { expiration: 3600 }));
    };
    CachedStorageStrategy.prototype.setItem = function (key, value, options) {
        var _this = this;
        return this.remoteStrategy
            .setItem(key, value, options)
            .then(function () { return _this._cacheItem(key, value, options); });
    };
    CachedStorageStrategy.prototype.removeItem = function (key, options) {
        var _this = this;
        return this.remoteStrategy
            .removeItem(key, options)
            .then(function () { return _this._cacheItem(key, DELETED, options); });
    };
    CachedStorageStrategy.prototype.getItem = function (key, options) {
        var _this = this;
        function throwIfDeletedOrReturn(value) {
            if (value === DELETED) {
                throw constants_1.NOT_FOUND;
            }
            return value;
        }
        return this.localStrategy
            .getItem(key, options)
            .catch(function () { return _this._getRemoteAndCache(key, options); })
            .then(throwIfDeletedOrReturn);
    };
    CachedStorageStrategy.prototype.getAllItems = function (options) {
        var _this = this;
        return this.remoteStrategy
            .getAllItems(options)
            .then(function (items) {
            return Promise.all(Object.keys(items).map(function (key) {
                return _this._cacheItem(key, items[key], options);
            })).then(function () { return items; });
        });
    };
    CachedStorageStrategy.prototype._getRemoteAndCache = function (key, options) {
        var _this = this;
        return this.remoteStrategy
            .getItem(key, options)
            .then(function (value) {
            return _this._cacheItem(key, value, options).then(function () { return value; });
        })
            .catch(function (e) {
            if (e === constants_1.NOT_FOUND) {
                return _this._cacheItem(key, DELETED, options).then(function () {
                    throw e;
                });
            }
            throw e;
        });
    };
    return CachedStorageStrategy;
}(base_storage_1.BaseStorage));
exports.CachedStorageStrategy = CachedStorageStrategy;
// we don't support local strategy in case we cannot
// identify the user - mainly on viewer flow.
function shouldIgnoreCache(remoteStrategy) {
    return (remoteStrategy &&
        remoteStrategy.constructor === wix_storage_1.WixStorageStrategy &&
        wix_storage_1.getUserId() === '');
}
//# sourceMappingURL=cached-storage.js.map