"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LocalStorageStrategy = void 0;
var tslib_1 = require("tslib");
var base_storage_1 = require("../base-storage");
var consent_policy_1 = require("../consent-policy");
var constants_1 = require("../utils/constants");
var local_storage_cleaner_1 = require("../utils/local-storage-cleaner");
var record_utils_1 = require("../utils/record-utils");
var LocalStorageStrategy = /** @class */ (function (_super) {
    tslib_1.__extends(LocalStorageStrategy, _super);
    function LocalStorageStrategy() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    LocalStorageStrategy.prototype.setItem = function (key, value, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var serializedKey, serializedValue;
            return tslib_1.__generator(this, function (_a) {
                serializedKey = getCacheKey(key, options);
                serializedValue = serializeData(value, options);
                // not mandatory, yet (gradual enforcement)
                if (options.category) {
                    consent_policy_1.verifyConsentPolicy(options.category);
                }
                try {
                    localStorage.setItem(serializedKey, serializedValue);
                }
                catch (e) {
                    local_storage_cleaner_1.localStorageCleaner(serializedKey.length + serializedValue.length);
                    localStorage.setItem(serializedKey, serializedValue);
                }
                return [2 /*return*/];
            });
        });
    };
    LocalStorageStrategy.prototype.getItem = function (key, options) {
        var fullKey = getCacheKey(key, options);
        var data;
        try {
            data = localStorage.getItem(fullKey);
        }
        catch (e) {
            return Promise.reject(constants_1.LOCAL_STORAGE_UNSUPPORTED);
        }
        data = data && record_utils_1.deserializeData(data);
        if (data && !record_utils_1.isExpired(data)) {
            updateAccessTime(fullKey, data);
            return Promise.resolve(data.value);
        }
        else {
            return Promise.reject(constants_1.NOT_FOUND);
        }
    };
    LocalStorageStrategy.prototype.removeItem = function (key, options) {
        key = getCacheKey(key, options);
        localStorage.removeItem(key);
        return Promise.resolve();
    };
    LocalStorageStrategy.prototype.getAllItems = function (options) {
        var prefix = getCachePrefix(options);
        var items = {};
        record_utils_1.getCacheRecords(prefix).forEach(function (record) {
            if (!record_utils_1.isExpired(record)) {
                items[record.key] = record.value;
            }
        });
        return Promise.resolve(items);
    };
    return LocalStorageStrategy;
}(base_storage_1.BaseStorage));
exports.LocalStorageStrategy = LocalStorageStrategy;
function getCacheKey(key, options) {
    return getCachePrefix(options) + key;
}
function stringify(obj) {
    return typeof obj === 'string' ? obj : JSON.stringify(obj);
}
function getCachePrefix(options) {
    return ([constants_1.STORAGE_PREFIX, options.namespace, stringify(options.scope)]
        .filter(function (x) { return x; })
        .join(constants_1.PREFIX_SEPARATOR) + constants_1.KEY_SEPARATOR);
}
function serializeData(value, options) {
    var data = {
        lastUsed: Date.now(),
        createdAt: options.createdAt || Date.now(),
        expiration: options.expiration,
        value: value,
    };
    return JSON.stringify(data);
}
function updateAccessTime(fullKey, data) {
    var expiration = data.expiration, createdAt = data.createdAt;
    localStorage.setItem(fullKey, serializeData(data.value, { expiration: expiration, createdAt: createdAt }));
}
//# sourceMappingURL=local-storage.js.map